/*************************************************************************/
/*                                                                       */
/* Program: rdsnot.c                                                     */
/*                                                                       */
/* Random Dot Stereogram program                                         */
/* =============================                                         */
/*                                                                       */
/* This program calculates a Random Dot Stereogram image.                */
/* This is an image with black dots on a white background                */
/* which looks noisy. But if you focus your eyes behind                  */
/* the image plane a 3-dimensional image will appear.                    */
/*                                                                       */
/* The program use Interval Bisection to perform ray-tracing.            */
/* (Newton Raphson is commented away for practical reasons.)             */
/*                                                                       */
/* Algorithm and program by: Stefan Spaennare, Lund, Sweden              */
/* E-mail:                   stefan@spaennare.se                         */
/* First version:            October 1992                                */
/* Latest update:            2005-07-23                                  */
/*                                                                       */
/*************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <graphics.h>


double pi=3.14159265358979323;


double p0,p1,p2,p3,p4;


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   long j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */



double f(x,y)
double x,y;
{
   double a,b,b1,b2,r,r0,z;
   
   a=p3*x;
   b=-p4*y;

   r=sqrt(a*a+b*b);

   z=p0;


   if ((r < 20.0) && (r > 10.0)) {
      z=p1;
   }

  if (
     ((-70<a) && (-60>a) && (20>b) && (-20<b)) ||
     ((-40<a) && (-30>a) && (20>b) && (-20<b)) ||
     ((30<a) && (70>a) && (20>b) && (10<b)) ||
     ((45<a) && (55>a) && (10>b) && (-20<b))) {
     z=p1;
  }
      
  if ((-60<a) && (-40>a)) {
     b1=-1.375*(a+60.0)+20.0;
     b2=-1.375*(a+60.0)+7.5;
     
     if ((b1>b) && (b2<b)) {
        z=p1;
     }
  }

  return(z); 

} /* f */


double fl(x,y,d,d2,de,xl)
double x,y,d,d2,de,xl;
{
   return(d/(xl+de)*(x-xl)-d2-f(x,y));
} /* fl */


double fr(x,y,d,d2,de,xr)
double x,y,d,d2,de,xr;
{
   return(d/(xr-de)*(x-xr)-d2-f(x,y));
} /* fr */



void main()
{


double x0,y0,x,y,z,x2,xr,yr,xl,yl;
double d,d2,de,ax,epsih,eps,eps0,s,u,u2,q,a,b,n,nf,alfa,h; 
double flprim,frprim,r;

long i,j,k,xsr,ysr,xsl,ysl,xs,ys,kmax,rows,size;
long col,bgcol,seed,idum;

int gdriver = DETECT, gmode;

clock_t time1,timediff;

clrscr();
printf("Random dot sterogram\n");
printf("\n");
printf("Input background colour (0-15) (15): ");
scanf("%D",&bgcol);
printf("Input dot colour (0-15) (0): ");
scanf("%D",&col);
printf("Input depth of 3D object (-10.0): ");
scanf("%lf",&p1);
clrscr();

initgraph(&gdriver,&gmode, "");

randomize();

p0=0.0;
p2=70.0;
p3=0.7;
p4=0.5;


d=200.0;
s=181.0;
size=480;

de=35.0;
ax=12.0;
d2=d*ax/(de-ax);
u=(double)(size);
u2=u/2.0;
q=(u-1)/s;
n=52428.0/(512.0*512.0)*u*u;
nf=s/(2.0*ax);
rows=(int)(n/nf);

/*
epsih=0.01;
*/
epsih=1.0e-14;
eps=1.0e-15;
eps0=sqrt(eps);
kmax=10;

seed=3;
idum=-abs(seed)-1;

setcolor(bgcol);

for (i=0; i<size; i++) {
   line(i,0,i,size);
} /* for i */


time1=clock();

for (i=0; i<rows; i++) {

   x0=s*(ran2(&idum)-0.5);
   y0=s*(ran2(&idum)-0.5);
   xs=(long)(q*x0+u2+0.5);
   ys=(long)(q*y0+u2+0.5);
   putpixel(xs,ys,0);


   y=(d+d2)/d*y0;

   xr=x0;
   x=x0;
   while (x < (0.6*s+2.0*ax)) {

      xl=xr;
      a=-s;
      b=s;
      x=1.0;

      while ((fabs(a-b)) > (fabs(epsih*x))) { 
         x=(a+b)/2.0;
         if ((fl(a,y,d,d2,de,xl)*
              fl(x,y,d,d2,de,xl)) > 0.0) {
            a=x;
         }
         else {
            b=x;
         }
      } /* while */

     /*
      h=fabs(x)*eps0+eps0;
      k=0;
      do {
          x2=x;
          flprim=(fl(x+h,y,d,d2,de,xl)-
                  fl(x-h,y,d,d2,de,xl))/(2.0*h);
          x=x-fl(x,y,d,d2,de,xl)/flprim; 
	  k++;
      } while (((fabs(x-x2)) > (fabs(x*eps))) && (k < kmax));
     */

      z=f(x,y);
      alfa=-(z+d2)/(z+d+d2);
      xr=x+(x-de)*alfa;
      xsr=(long)(q*xr+u2+0.5);
      ysr=(long)(q*y0+u2+0.5);

      if ((xsr >=0) && (xsr < size)) {
	 if ((ysr >=0) && (ysr < size)) {
	    putpixel(xsr,ysr,col);
         }
      }
   } /* while */

   xl=x0;
   x=x0;     
   while (x > (-0.6*s-2.0*ax)) {
      xr=xl;
      a=-s;
      b=s;
      x=1.0;
      while ((fabs(a-b)) > (fabs(epsih*x))) { 
         x=(a+b)/2.0;
         if ((fr(a,y,d,d2,de,xr)*
              fr(x,y,d,d2,de,xr)) > 0.0) {
            a=x;
         }
         else {
            b=x;
         }
      } /* while */

     /*
      h=fabs(x)*eps0+eps0;
      k=0;
      do {
          x2=x;
          frprim=(fr(x+h,y,d,d2,de,xl)-
                  fr(x-h,y,d,d2,de,xl))/(2.0*h);
          x=x-fr(x,y,d,d2,de,xr)/frprim; 
	  k++;
      } while (((fabs(x-x2)) > (fabs(x*eps))) && (k < kmax));
     */

      z=f(x,y);
      alfa=-(z+d2)/(z+d+d2);
      xl=x+(x+de)*alfa;
      xsl=(long)(q*xl+u2+0.5);
      ysl=(long)(q*y0+u2+0.5);

      if ((xsl >=0) && (xsl < size)) {
         if ((ysl >=0) && (ysl < size)) {
	    putpixel(xsl,ysl,col);
	 }
      }
   } /* while */

} /* for i */

timediff=clock()-time1;


getch();

closegraph();

} /* End */
